// Time-stamp: <2001-08-22 19:41:19 dvz90620> (Dmitri Zaykin)
//
// normal, beta random deviates
//

#ifndef DZ_GAUSS_H
#define DZ_GAUSS_H

#include <math.h>
#include "marsa.h"

class Gauss : public Rndm {
    double v1, v2, s;
    int phase;
public:
    Gauss(Unlo i1, Unlo i2, Unlo i3, Unlo i4, Unlo i5, Unlo i6)
            : Rndm(i1, i2, i3, i4, i5, i6) {
        v1=v2=s=phase=0;
    }
    Gauss(Unlo i1) : Rndm(i1) {
        v1=v2=s=phase=0;
    }
    Gauss() : Rndm() { v1=v2=s=phase=0; }

    virtual void Init(Unlo i1) {
	Rndm::Init(i1);
        v1=v2=s=phase=0;
    }
    virtual void Init(Unlo i1, Unlo i2, Unlo i3, Unlo i4, Unlo i5, Unlo i6) {
	Rndm::Init(i1, i2, i3, i4, i5, i6);
        v1=v2=s=phase=0;
    }

        // Box-Muller
    double gss (double m=0, double stddev=1) {
        if(stddev <= 0) return m;
        double x;
        if(phase == 0) {
            do {
                v1 = 2*Uni()-1, v2 = 2*Uni()-1;
                s = v1*v1 + v2*v2;
            } while(s >= 1 || s == 0);
            x = v1 * sqrt(-2 * log(s) / s);
        }
        else x = v2 * sqrt(-2 * log(s) / s);
        phase = 1 - phase;
        return stddev*x + m;
    }
    
    double rbeta(double a, double b) { // rejection method (might be slow)
        double x1, x2, w;
        do {
            x1 = pow(Uni(), 1.0/a);
            x2 = pow(Uni(), 1.0/b);
        } while ((w=x1+x2) > 1.0);
        return x1/w;
    }

    double beta(double a, double b) { 
        if(a == 1 && b == 1) return Uni();
        double x[2] = { Gengam(a, *this), Gengam(b, *this) };
        return x[0] / (x[0] + x[1]);
    }

    double bound_beta(double a, double b, double mn, double mx) {
        return mn + beta(a,b)*(mx-mn);
    }
    
    Unlo beta(double a, double b, Unlo n) {
        return (Unlo)(n * beta(a,b));
    }
};

int bnldev(Gauss& rand_u01, float pp, int n);
int poidev (Gauss& rand_u01, double xm);
double TruncBeta(Gauss& rn, double a, double b, double mn, double mx);

#endif // DZ_GAUSS_H
